@@ -306,12 +306,56 @@ void
 neworthconn_update_data(NewOrthConn *orth)
 {
   int i;
-  Object *obj = (Object *)orth;
+  DiaObject *obj = (DiaObject *)orth;
+  Point *points;
+  ConnectionPoint *start_cp;
+  ConnectionPoint *end_cp;
+
+  obj->position = orth->points[0];
+
+  /* During startup, handles may not have been setup yet, so do so
+   * temporarily to be able to get the last handle connection.
+   *
+   * [This looks like avoiding the symptom but not healing the
+   *  root cause, but it appears to work for orth_conn.c since
+   *  a while so do it here as well ...                   --hb]
+   */
+  adjust_handle_count_to(orth, orth->numpoints-1);
+
+  start_cp = orth->handles[0]->connected_to;
+  end_cp = orth->handles[orth->numpoints-2]->connected_to;
 
   if (!orth->points) {
     g_warning("This NewOrthConn object is very sick !");
     return;
   }
+
+  points = orth->points;
+  if (connpoint_is_autogap(start_cp) || 
+      connpoint_is_autogap(end_cp)) {
+    Point* new_points = g_new(Point, orth->numpoints);
+    int i;
+    for (i = 0; i < orth->numpoints; i++) {
+      new_points[i] = points[i];
+    }
+
+    if (connpoint_is_autogap(start_cp)) {
+      new_points[0] = calculate_object_edge(&start_cp->pos, &new_points[1],
+					    start_cp->object);
+      printf("Moved start to %f, %f\n",
+	     new_points[0].x, new_points[0].y);
+    }
+    if (connpoint_is_autogap(end_cp)) {
+      new_points[orth->numpoints-1] =
+	calculate_object_edge(&end_cp->pos, &new_points[orth->numpoints-2],
+			      end_cp->object);
+      printf("Moved end to %f, %f\n",
+	     new_points[orth->numpoints-1].x, new_points[orth->numpoints-1].y);
+    }
+    g_free(points);
+    orth->points = new_points;
+  }
+
   obj->position = orth->points[0];
 
   adjust_handle_count_to(orth,orth->numpoints-1);
@@ -406,14 +450,14 @@ neworthconn_can_add_segment(NewOrthConn *orth, Point *clickedpoint)
 
 /* Needs to have at least 2 handles. 
    The handles are stored in order in the NewOrthConn, but need
-   not be stored in order in the Object.handles array. This
+   not be stored in order in the DiaObject.handles array. This
    is so that derived object can do what they want with
-   Object.handles. */
+   DiaObject.handles. */
 
 void
 neworthconn_init(NewOrthConn *orth, Point *startpoint)
 {
-  Object *obj;
+  DiaObject *obj;
 
   obj = &orth->object;
 
@@ -461,7 +505,7 @@ void
 neworthconn_copy(NewOrthConn *from, NewOrthConn *to)
 {
   int i,rcc;
-  Object *toobj, *fromobj;
+  DiaObject *toobj, *fromobj;
 
   toobj = &to->object;
   fromobj = &from->object;
@@ -514,11 +558,11 @@ neworthconn_destroy(NewOrthConn *orth)
 static void 
 place_handle_by_swapping(NewOrthConn *orth, int index, Handle *handle)
 {
-  Object *obj;
+  DiaObject *obj;
   Handle *tmp;
   int j;
 
-  obj = (Object *)orth;
+  obj = (DiaObject *)orth;
   if (obj->handles[index] == handle)
     return; /* Nothing to do */
 
@@ -566,7 +610,7 @@ neworthconn_load(NewOrthConn *orth, ObjectNode obj_node) /* NOTE: Does object_in
   DataNode data;
   int n;
   
-  Object *obj = &orth->object;
+  DiaObject *obj = &orth->object;
 
   object_load(obj, obj_node);
 
@@ -663,7 +707,7 @@ neworthconn_delete_segment(NewOrthConn *orth, Point *clickedpoint)
 				      orth->handles[segment+1]);
   }
 
-  change->apply(change, (Object *)orth);
+  change->apply(change, (DiaObject *)orth);
   
   return change;
 }
@@ -710,7 +754,7 @@ neworthconn_add_segment(NewOrthConn *orth, Point *clickedpoint)
 				      handle2);
   }
 
-  change->apply(change, (Object *)orth);
+  change->apply(change, (DiaObject *)orth);
 
   return change;
 }
@@ -814,7 +858,7 @@ endsegment_change_free(struct EndSegmentChange *change)
 }
 
 static void
-endsegment_change_apply(struct EndSegmentChange *change, Object *obj)
+endsegment_change_apply(struct EndSegmentChange *change, DiaObject *obj)
 {
   NewOrthConn *orth = (NewOrthConn *)obj;
 
@@ -862,7 +906,7 @@ endsegment_change_apply(struct EndSegmentChange *change, Object *obj)
 }
 
 static void
-endsegment_change_revert(struct EndSegmentChange *change, Object *obj)
+endsegment_change_revert(struct EndSegmentChange *change, DiaObject *obj)
 {
   NewOrthConn *orth = (NewOrthConn *)obj;
 
@@ -956,7 +1000,7 @@ midsegment_change_free(struct MidSegmentChange *change)
 }
 
 static void
-midsegment_change_apply(struct MidSegmentChange *change, Object *obj)
+midsegment_change_apply(struct MidSegmentChange *change, DiaObject *obj)
 {
   NewOrthConn *orth = (NewOrthConn *)obj;
   int seg;
@@ -999,7 +1043,7 @@ midsegment_change_apply(struct MidSegmentChange *change, Object *obj)
 }
 
 static void
-midsegment_change_revert(struct MidSegmentChange *change, Object *obj)
+midsegment_change_revert(struct MidSegmentChange *change, DiaObject *obj)
 {
   NewOrthConn *orth = (NewOrthConn *)obj;
   
