@@ -22,14 +22,18 @@
 #include <assert.h>
 #include <math.h>
 #include <string.h> /* memcpy() */
+#include <glib.h>
 
 #include "orth_conn.h"
 #include "message.h"
 #include "diamenu.h"
 #include "handle.h"
+#include "diarenderer.h"
+#include "autoroute.h"
 
 static void place_handle_by_swapping(OrthConn *orth, 
                                      int index, Handle *handle);
+static ObjectChange *orthconn_set_autorouting(OrthConn *orth, gboolean on);
 
 enum change_type {
   TYPE_ADD_SEGMENT,
@@ -91,6 +95,14 @@ struct EndSegmentChange {
 			  remove segment */
 };
 
+static ObjectChange*
+autoroute_create_change(OrthConn *orth, gboolean on);
+
+struct AutorouteChange {
+  ObjectChange obj_change;
+  gboolean on;
+  Point *points;
+};
 
 static void set_midpoint(Point *point, OrthConn *orth, int segment)
 {
@@ -149,16 +161,23 @@ static int get_segment_nr(OrthConn *orth, Point *point, real max_dist)
 }
 
 
-void
+ObjectChange *
 orthconn_move_handle(OrthConn *orth, Handle *handle,
-		     Point *to, HandleMoveReason reason)
+		     Point *to, ConnectionPoint *cp,
+		     HandleMoveReason reason, ModifierKeys modifiers)
 {
   int n;
   int handle_nr;
-  
+  DiaObject *obj = (DiaObject *)orth;
+  ObjectChange *change = NULL;
+
   switch(handle->id) {
   case HANDLE_MOVE_STARTPOINT:
     orth->points[0] = *to;
+    if (orth->autorouting &&
+	autoroute_layout_orthconn(orth, cp,
+				  obj->handles[1]->connected_to))
+      break;
     switch (orth->orientation[0]) {
     case HORIZONTAL:
       orth->points[1].y = to->y;
@@ -171,6 +190,10 @@ orthconn_move_handle(OrthConn *orth, Handle *handle,
   case HANDLE_MOVE_ENDPOINT:
     n = orth->numpoints - 1;
     orth->points[n] = *to;
+    if (orth->autorouting &&
+	autoroute_layout_orthconn(orth, obj->handles[0]->connected_to,
+				  cp))
+      break;
     switch (orth->orientation[n-1]) {
     case HORIZONTAL:
       orth->points[n-1].y = to->y;
@@ -178,12 +201,13 @@ orthconn_move_handle(OrthConn *orth, Handle *handle,
     case VERTICAL:
       orth->points[n-1].x = to->x;
       break;
-    } 
+    }
     break;
   case HANDLE_MIDPOINT:
     n = orth->numpoints - 1;
     handle_nr = get_handle_nr(orth, handle);
-
+    if (orth->autorouting)
+      change = orthconn_set_autorouting(orth, FALSE);
     switch (orth->orientation[handle_nr]) {
     case HORIZONTAL:
       orth->points[handle_nr].y = to->y;
@@ -199,9 +223,11 @@ orthconn_move_handle(OrthConn *orth, Handle *handle,
     message_error("Internal error in orthconn_move_handle.\n");
     break;
   }
+
+  return change;
 }
 
-void
+ObjectChange *
 orthconn_move(OrthConn *orth, Point *to)
 {
   Point p;
@@ -214,6 +240,7 @@ orthconn_move(OrthConn *orth, Point *to)
   for (i=1;i<orth->numpoints;i++) {
     point_add(&orth->points[i], &p);
   }
+  return NULL;
 }
 
 real
@@ -274,15 +301,60 @@ void
 orthconn_update_data(OrthConn *orth)
 {
   int i;
-  Object *obj = (Object *)orth;
-  
+  DiaObject *obj = (DiaObject *)orth;
+  Point *points;
+  ConnectionPoint *start_cp;
+  ConnectionPoint *end_cp;
+
+  obj->position = orth->points[0];
+
+  /* During startup, handles may not have been setup yet, so do so
+   * temporarily to be able to get the last handle connection.
+   */
+  adjust_handle_count_to(orth, orth->numpoints-1);
+
+  start_cp = orth->handles[0]->connected_to;
+  end_cp = orth->handles[orth->numpoints-2]->connected_to;
+
   if (!orth->points) {
     g_warning("very sick OrthConn object...");
     return;
   }
+
+  points = orth->points;
+  if (!orth->autorouting &&
+      (connpoint_is_autogap(start_cp) || 
+       connpoint_is_autogap(end_cp))) {
+    Point* new_points = g_new(Point, orth->numpoints);
+    int i;
+    for (i = 0; i < orth->numpoints; i++) {
+      new_points[i] = points[i];
+    }
+
+    if (connpoint_is_autogap(start_cp)) {
+      new_points[0] = calculate_object_edge(&start_cp->pos, &new_points[1],
+					    start_cp->object);
+      /*
+      printf("Moved start to %f, %f\n",
+	     new_points[0].x, new_points[0].y);
+      */
+    }
+    if (connpoint_is_autogap(end_cp)) {
+      new_points[orth->numpoints-1] =
+	calculate_object_edge(&end_cp->pos, &new_points[orth->numpoints-2],
+			      end_cp->object);
+      /*
+      printf("Moved end to %f, %f\n",
+	     new_points[orth->numpoints-1].x, new_points[orth->numpoints-1].y);
+      */
+    }
+    g_free(points);
+    orth->points = new_points;
+  }
+
   obj->position = orth->points[0];
 
-  adjust_handle_count_to(orth,orth->numpoints-1);
+  adjust_handle_count_to(orth, orth->numpoints-1);
 
   /* Make sure start-handle is first and end-handle is second. */
   place_handle_by_swapping(orth, 0, orth->handles[0]);
@@ -308,7 +380,7 @@ orthconn_update_boundingbox(OrthConn *orth)
 }
 
 void
-orthconn_simple_draw(OrthConn *orth, Renderer *renderer, real width)
+orthconn_simple_draw(OrthConn *orth, DiaRenderer *renderer, real width)
 {
   Point *points;
   
@@ -319,15 +391,18 @@ orthconn_simple_draw(OrthConn *orth, Renderer *renderer, real width)
     g_warning("very sick OrthConn object...");
     return;
   }
+
+  /* When not autorouting, need to take gap into account here. */
   points = &orth->points[0];
   
-  renderer->ops->set_linewidth(renderer, width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, width);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_polyline(renderer, points, orth->numpoints,
-			       &color_black);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_polyline(renderer, points, 
+						  orth->numpoints,
+						  &color_black);
 }
 
 
@@ -371,14 +446,14 @@ orthconn_can_add_segment(OrthConn *orth, Point *clickedpoint)
 
 /* Needs to have at least 2 handles. 
    The handles are stored in order in the OrthConn, but need
-   not be stored in order in the Object.handles array. This
+   not be stored in order in the DiaObject.handles array. This
    is so that derived object can do what they want with
-   Object.handles. */
+   DiaObject.handles. */
 
 void
 orthconn_init(OrthConn *orth, Point *startpoint)
 {
-  Object *obj;
+  DiaObject *obj;
 
   obj = &orth->object;
 
@@ -406,6 +481,8 @@ orthconn_init(OrthConn *orth, Point *startpoint)
   setup_endpoint_handle(orth->handles[2], HANDLE_MOVE_ENDPOINT);
   obj->handles[2] = orth->handles[2];
 
+  orth->autorouting = TRUE;
+
   /* Just so we have some position: */
   orth->points[0] = *startpoint;
   orth->points[1].x = startpoint->x;
@@ -422,11 +499,42 @@ orthconn_init(OrthConn *orth, Point *startpoint)
   orthconn_update_data(orth);
 }
 
+/** This function does *not* set up handles */
+void
+orthconn_set_points(OrthConn *orth, int num_points, Point *points) 
+{
+  int i;
+  gboolean horiz;
+
+  orth->numpoints = num_points;
+
+  if (orth->points)
+    g_free(orth->points);
+
+  orth->points = g_malloc((orth->numpoints)*sizeof(Point));
+
+  for (i=0;i<orth->numpoints;i++) {
+    orth->points[i] = points[i];
+  }
+
+  /* Set up the orientation array. */
+  /* Maybe we could get rid of this array altogether? */
+  orth->numorient = orth->numpoints-1;
+  if (orth->orientation) g_free(orth->orientation);
+  orth->orientation = g_new(Orientation, orth->numorient);
+  horiz = (fabs(orth->points[0].y-orth->points[1].y) < 0.00001);
+  for (i = 0; i < orth->numorient; i++) {
+    if (horiz) orth->orientation[i] = HORIZONTAL;
+    else orth->orientation[i] = VERTICAL;
+    horiz = !horiz;
+  }
+}
+
 void
 orthconn_copy(OrthConn *from, OrthConn *to)
 {
   int i;
-  Object *toobj, *fromobj;
+  DiaObject *toobj, *fromobj;
 
   toobj = &to->object;
   fromobj = &from->object;
@@ -442,6 +550,7 @@ orthconn_copy(OrthConn *from, OrthConn *to)
     to->points[i] = from->points[i];
   }
 
+  to->autorouting = from->autorouting;
   to->orientation = g_malloc0((to->numpoints-1)*sizeof(Orientation));
   to->numhandles = from->numhandles;
   to->handles = g_malloc0((to->numpoints-1)*sizeof(Handle *));
@@ -475,11 +584,11 @@ orthconn_destroy(OrthConn *orth)
 static void 
 place_handle_by_swapping(OrthConn *orth, int index, Handle *handle)
 {
-  Object *obj;
+  DiaObject *obj;
   Handle *tmp;
   int j;
 
-  obj = (Object *)orth;
+  obj = (DiaObject *)orth;
   if (obj->handles[index] == handle)
     return; /* Nothing to do */
 
@@ -517,6 +626,8 @@ orthconn_save(OrthConn *orth, ObjectNode obj_node)
   for (i=0;i<orth->numpoints-1;i++) {
     data_add_enum(attr, orth->orientation[i]);
   }
+
+  data_add_boolean(new_attribute(obj_node, "autorouting"), orth->autorouting);
 }
 
 void
@@ -526,11 +637,16 @@ orthconn_load(OrthConn *orth, ObjectNode obj_node) /* NOTE: Does object_init() *
   AttributeNode attr;
   DataNode data;
   int n;
+  int version = 0;
   
-  Object *obj = &orth->object;
+  DiaObject *obj = &orth->object;
 
   object_load(obj, obj_node);
 
+  attr = object_find_attribute(obj_node, "version");
+  if (attr != NULL)
+    version = attribute_num_data(attr);
+
   attr = object_find_attribute(obj_node, "orth_points");
 
   if (attr != NULL)
@@ -558,6 +674,15 @@ orthconn_load(OrthConn *orth, ObjectNode obj_node) /* NOTE: Does object_init() *
     data = data_next(data);
   }
 
+  orth->autorouting = TRUE;
+  attr = object_find_attribute(obj_node, "autorouting");
+  if (attr != NULL)
+    orth->autorouting = data_boolean(attribute_first_data(attr));
+  else if (version == 0) {
+    /* Version 0 orthconns have no autorouting. */
+    orth->autorouting = FALSE;
+  }
+
   orth->handles = g_malloc0((orth->numpoints-1)*sizeof(Handle *));
 
   orth->handles[0] = g_new(Handle, 1);
@@ -623,7 +748,7 @@ orthconn_delete_segment(OrthConn *orth, Point *clickedpoint)
 				      orth->handles[segment+1]);
   }
 
-  change->apply(change, (Object *)orth);
+  change->apply(change, (DiaObject *)orth);
   
   return change;
 }
@@ -670,11 +795,26 @@ orthconn_add_segment(OrthConn *orth, Point *clickedpoint)
 				      handle2);
   }
 
-  change->apply(change, (Object *)orth);
+  change->apply(change, (DiaObject *)orth);
 
   return change;
 }
 
+
+/* Set autorouting on or off.  If setting on, try to autoroute and
+ * return the changes from that.
+ */
+static ObjectChange *
+orthconn_set_autorouting(OrthConn *conn, gboolean on)
+{
+  DiaObject *obj = (DiaObject *)conn;
+  ObjectChange *change;
+
+  change = autoroute_create_change(conn, on);
+  change->apply(change, obj);
+  return change;
+}
+
 static void
 delete_point(OrthConn *orth, int pos)
 {
@@ -755,8 +895,6 @@ insert_handle(OrthConn *orth, int segment,
   orth->numhandles = orth->numpoints-1;
 }
 
-
-
 static void
 endsegment_change_free(struct EndSegmentChange *change)
 {
@@ -769,7 +907,7 @@ endsegment_change_free(struct EndSegmentChange *change)
 }
 
 static void
-endsegment_change_apply(struct EndSegmentChange *change, Object *obj)
+endsegment_change_apply(struct EndSegmentChange *change, DiaObject *obj)
 {
   OrthConn *orth = (OrthConn *)obj;
 
@@ -811,7 +949,7 @@ endsegment_change_apply(struct EndSegmentChange *change, Object *obj)
 }
 
 static void
-endsegment_change_revert(struct EndSegmentChange *change, Object *obj)
+endsegment_change_revert(struct EndSegmentChange *change, DiaObject *obj)
 {
   OrthConn *orth = (OrthConn *)obj;
   
@@ -895,7 +1033,7 @@ midsegment_change_free(struct MidSegmentChange *change)
 }
 
 static void
-midsegment_change_apply(struct MidSegmentChange *change, Object *obj)
+midsegment_change_apply(struct MidSegmentChange *change, DiaObject *obj)
 {
   OrthConn *orth = (OrthConn *)obj;
   change->applied = 1;
@@ -924,7 +1062,7 @@ midsegment_change_apply(struct MidSegmentChange *change, Object *obj)
 }
 
 static void
-midsegment_change_revert(struct MidSegmentChange *change, Object *obj)
+midsegment_change_revert(struct MidSegmentChange *change, DiaObject *obj)
 {
   OrthConn *orth = (OrthConn *)obj;
   
@@ -977,6 +1115,79 @@ midsegment_create_change(OrthConn *orth, enum change_type type,
   return (ObjectChange *)change;
 }
 
+static void
+autoroute_change_free(struct AutorouteChange *change)
+{
+  g_free(change->points);
+}
 
+static void
+autoroute_change_apply(struct AutorouteChange *change, DiaObject *obj)
+{
+  OrthConn *orth = (OrthConn*)obj;
 
+  if (change->on) {
+    orth->autorouting = TRUE;
+    autoroute_layout_orthconn(orth, obj->handles[0]->connected_to,
+			      obj->handles[1]->connected_to);
+  } else {
+    orth->autorouting = FALSE;
+    orthconn_set_points(orth, orth->numpoints, change->points);
+  }
+}
 
+static void
+autoroute_change_revert(struct AutorouteChange *change, DiaObject *obj)
+{
+  OrthConn *orth = (OrthConn*)obj;
+
+  if (change->on) {
+    orth->autorouting = FALSE;
+    orthconn_set_points(orth, orth->numpoints, change->points);
+  } else {
+    orth->autorouting = TRUE;
+    autoroute_layout_orthconn(orth, obj->handles[0]->connected_to,
+			      obj->handles[1]->connected_to);
+  }
+}
+
+static ObjectChange *
+autoroute_create_change(OrthConn *orth, gboolean on)
+{
+  struct AutorouteChange *change;
+  int i;
+
+  change = g_new(struct AutorouteChange, 1);
+
+  change->obj_change.apply = (ObjectChangeApplyFunc) autoroute_change_apply;
+  change->obj_change.revert = (ObjectChangeRevertFunc) autoroute_change_revert;
+  change->obj_change.free = (ObjectChangeFreeFunc) autoroute_change_free;
+
+  change->on = on;
+  change->points = g_new(Point, orth->numpoints);
+  for (i = 0; i < orth->numpoints; i++)
+    change->points[i] = orth->points[i];
+
+  return (ObjectChange *)change;
+}
+
+ObjectChange *
+orthconn_toggle_autorouting_callback(DiaObject *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  /* This is kinda hackish.  Since we can't see the menu item, we have to
+   * assume that we're right about toggling and just send !orth->autorouting.
+   */
+  change = orthconn_set_autorouting((OrthConn*)obj, 
+				    !((OrthConn*)obj)->autorouting);
+  orthconn_update_data((OrthConn *)obj);
+  return change;
+}
+
+void
+orthconn_update_object_menu(OrthConn *orth, Point *clicked,
+			    DiaMenuItem *object_menu_items)
+{
+  object_menu_items[0].active = DIAMENU_ACTIVE|DIAMENU_TOGGLE|
+    (orth->autorouting?DIAMENU_TOGGLE_ON:0);
+}
