@@ -67,20 +67,21 @@ typedef struct _Action {
   ConnPointLine *cps; /* aaahrg ! again one ! */
 } Action;
 
-static void action_move_handle(Action *action, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void action_move(Action *action, Point *to);
+static ObjectChange* action_move_handle(Action *action, Handle *handle,
+					Point *to, ConnectionPoint *cp,
+					HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* action_move(Action *action, Point *to);
 static void action_select(Action *action, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void action_draw(Action *action, DiaRenderer *renderer);
-static Object *action_create(Point *startpoint,
+static DiaObject *action_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real action_distance_from(Action *action, Point *point);
 static void action_update_data(Action *action);
 static void action_destroy(Action *action);
-static Object *action_load(ObjectNode obj_node, int version,
+static DiaObject *action_load(ObjectNode obj_node, int version,
 			       const char *filename);
 static PropDescription *action_describe_props(Action *action);
 static void action_get_props(Action *action, 
@@ -98,7 +99,7 @@ static ObjectTypeOps action_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType action_type =
+DiaObjectType action_type =
 {
   "GRAFCET - Action",   /* name */
   0,                         /* version */
@@ -197,9 +198,10 @@ action_select(Action *action, Point *clicked_point,
   text_grab_focus(action->text, &action->connection.object);
 }
 
-static void
+static ObjectChange*
 action_move_handle(Action *action, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		   Point *to, ConnectionPoint *cp,
+		   HandleMoveReason reason, ModifierKeys modifiers)
 {
   g_assert(action!=NULL);
   g_assert(handle!=NULL);
@@ -213,15 +215,18 @@ action_move_handle(Action *action, Handle *handle,
     point_sub(&to2,&action->connection.endpoints[0]);
     point_add(&to2,&action->connection.endpoints[1]);
     connection_move_handle(&action->connection, HANDLE_MOVE_ENDPOINT, 
-			   to, reason);
+			   to, NULL, reason, 0);
   }
 #endif
-  connection_move_handle(&action->connection, handle->id, to, reason);
+  connection_move_handle(&action->connection, handle->id, to, cp, 
+			 reason, modifiers);
   action_update_data(action);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 action_move(Action *action, Point *to)
 {
   Point start_to_end;
@@ -234,6 +239,8 @@ action_move(Action *action, Point *to)
   point_add(&endpoints[1], &start_to_end);
 
   action_update_data(action);
+
+  return NULL;
 }
 
 static void
@@ -245,7 +252,7 @@ action_update_data(Action *action)
   int i;
   real chunksize;
   Connection *conn = &action->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
 
   obj->position = conn->endpoints[0];
   connection_update_boundingbox(conn);
@@ -295,14 +302,18 @@ action_update_data(Action *action)
     p1.x = p2.x = x1;
 
     obj->connections[(2*i) + 2]->pos = p1;
+    obj->connections[(2*i) + 2]->directions = DIR_NORTH;
     obj->connections[(2*i) + 3]->pos = p2;
+    obj->connections[(2*i) + 3]->directions = DIR_SOUTH;
 
     x = x + chunksize + 2 * action->space_width; 
   }
   p1.y = p2.y = conn->endpoints[1].y;
   p1.x = left; p2.x = right;
   obj->connections[0]->pos = p1;
+  obj->connections[0]->directions = DIR_WEST;
   obj->connections[1]->pos = p2;
+  obj->connections[1]->directions = DIR_EAST;
 
   /* fix boundingbox for line_width: */
   action->labelbb.top -= ACTION_LINE_WIDTH/2;
@@ -380,7 +391,7 @@ action_draw(Action *action, DiaRenderer *renderer)
   renderer_ops->draw_rect(renderer,&ul,&br,&color_black);
 }
 
-static Object *
+static DiaObject *
 action_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -388,7 +399,7 @@ action_create(Point *startpoint,
 {
   Action *action;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   LineBBExtras *extra;
   Point defaultlen  = {1.0,0.0}, pos;
 
@@ -443,7 +454,7 @@ action_destroy(Action *action)
   connection_destroy(&action->connection);
 }
  
-static Object *
+static DiaObject *
 action_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&action_type,
