@@ -21,21 +21,20 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "polyshape.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "diamenu.h"
 #include "message.h"
 #include "properties.h"
 
-#include "pixmaps/polygon.xpm"
+#include "tool-icons.h"
 
 #include "create.h"
 
@@ -49,9 +48,6 @@ Fix crashes:)
 
 #define DEFAULT_WIDTH 0.15
 
-typedef struct _PolygonProperties PolygonProperties;
-typedef struct _PolygonDefaultsDialog PolygonDefaultsDialog;
-
 typedef struct _Polygon {
   PolyShape poly;
 
@@ -63,35 +59,25 @@ typedef struct _Polygon {
   real line_width;
 } Polygon;
 
-struct _PolygonProperties {
+static struct _PolygonProperties {
   gboolean show_background;
-};
+} default_properties = { TRUE };
 
-struct _PolygonDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-};
-
-
-static PolygonDefaultsDialog *polygon_defaults_dialog;
-static PolygonProperties default_properties;
-
-
-static void polygon_move_handle(Polygon *polygon, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void polygon_move(Polygon *polygon, Point *to);
+static ObjectChange* polygon_move_handle(Polygon *polygon, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* polygon_move(Polygon *polygon, Point *to);
 static void polygon_select(Polygon *polygon, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void polygon_draw(Polygon *polygon, Renderer *renderer);
-static Object *polygon_create(Point *startpoint,
+			      DiaRenderer *interactive_renderer);
+static void polygon_draw(Polygon *polygon, DiaRenderer *renderer);
+static DiaObject *polygon_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real polygon_distance_from(Polygon *polygon, Point *point);
 static void polygon_update_data(Polygon *polygon);
 static void polygon_destroy(Polygon *polygon);
-static Object *polygon_copy(Polygon *polygon);
+static DiaObject *polygon_copy(Polygon *polygon);
 
 static PropDescription *polygon_describe_props(Polygon *polygon);
 static void polygon_get_props(Polygon *polygon, GPtrArray *props);
@@ -99,32 +85,31 @@ static void polygon_set_props(Polygon *polygon, GPtrArray *props);
 
 static void polygon_save(Polygon *polygon, ObjectNode obj_node,
 			  const char *filename);
-static Object *polygon_load(ObjectNode obj_node, int version,
+static DiaObject *polygon_load(ObjectNode obj_node, int version,
 			     const char *filename);
 static DiaMenu *polygon_get_object_menu(Polygon *polygon, Point *clickedpoint);
 
-static GtkWidget *polygon_get_defaults(void);
-static void polygon_apply_defaults(void);
-
 static ObjectTypeOps polygon_type_ops =
 {
   (CreateFunc)polygon_create,   /* create */
   (LoadFunc)  polygon_load,     /* load */
   (SaveFunc)  polygon_save,      /* save */
-  (GetDefaultsFunc)   polygon_get_defaults,
-  (ApplyDefaultsFunc) polygon_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL,
 };
 
-static ObjectType polygon_type =
+static DiaObjectType polygon_type =
 {
   "Standard - Polygon",   /* name */
   0,                         /* version */
-  (char **) polygon_xpm,      /* pixmap */
+  (char **) polygon_icon,      /* pixmap */
   
-  &polygon_type_ops       /* ops */
+  &polygon_type_ops,       /* ops */
+  NULL, /* pixmap_file */
+  0 /* default_user_data */
 };
 
-ObjectType *_polygon_type = (ObjectType *) &polygon_type;
+DiaObjectType *_polygon_type = (DiaObjectType *) &polygon_type;
 
 
 static ObjectOps polygon_ops = {
@@ -144,7 +129,7 @@ static ObjectOps polygon_ops = {
 };
 
 static PropDescription polygon_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  POLYSHAPE_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
@@ -162,7 +147,7 @@ polygon_describe_props(Polygon *polygon)
 }
 
 static PropOffset polygon_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  POLYSHAPE_COMMON_PROPERTIES_OFFSETS,
   { "line_width", PROP_TYPE_REAL, offsetof(Polygon, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Polygon, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
@@ -187,60 +172,6 @@ polygon_set_props(Polygon *polygon, GPtrArray *props)
   polygon_update_data(polygon);
 }
 
-static void
-polygon_apply_defaults()
-{
-  default_properties.show_background = 
-    gtk_toggle_button_get_active(polygon_defaults_dialog->show_background);
-}
-
-static void
-init_default_values() {
-  static int defaults_initialized = 0;
-
-  if (!defaults_initialized) {
-    default_properties.show_background = 1;
-    defaults_initialized = 1;
-  }
-}
-
-static GtkWidget *
-polygon_get_defaults()
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *checkbox;
-
-  if (polygon_defaults_dialog == NULL) {
-  
-    init_default_values();
-
-    polygon_defaults_dialog = g_new(PolygonDefaultsDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    polygon_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Draw background"));
-    polygon_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show (vbox);
-  }
-
-  gtk_toggle_button_set_active(polygon_defaults_dialog->show_background, 
-			       default_properties.show_background);
-
-  return polygon_defaults_dialog->vbox;
-}
-
-
 static real
 polygon_distance_from(Polygon *polygon, Point *point)
 {
@@ -257,34 +188,40 @@ static int polygon_closest_segment(Polygon *polygon, Point *point) {
 
 static void
 polygon_select(Polygon *polygon, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   polyshape_update_data(&polygon->poly);
 }
 
-static void
+static ObjectChange*
 polygon_move_handle(Polygon *polygon, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(polygon!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  polyshape_move_handle(&polygon->poly, handle, to, reason);
+  polyshape_move_handle(&polygon->poly, handle, to, cp, reason, modifiers);
   polygon_update_data(polygon);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 polygon_move(Polygon *polygon, Point *to)
 {
   polyshape_move(&polygon->poly, to);
   polygon_update_data(polygon);
+
+  return NULL;
 }
 
 static void
-polygon_draw(Polygon *polygon, Renderer *renderer)
+polygon_draw(Polygon *polygon, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   PolyShape *poly = &polygon->poly;
   Point *points;
   int n;
@@ -292,19 +229,19 @@ polygon_draw(Polygon *polygon, Renderer *renderer)
   points = &poly->points[0];
   n = poly->numpoints;
 
-  renderer->ops->set_linewidth(renderer, polygon->line_width);
-  renderer->ops->set_linestyle(renderer, polygon->line_style);
-  renderer->ops->set_dashlength(renderer, polygon->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, polygon->line_width);
+  renderer_ops->set_linestyle(renderer, polygon->line_style);
+  renderer_ops->set_dashlength(renderer, polygon->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   if (polygon->show_background)
-    renderer->ops->fill_polygon(renderer, points, n, &polygon->inner_color);
+    renderer_ops->fill_polygon(renderer, points, n, &polygon->inner_color);
 
-  renderer->ops->draw_polygon(renderer, points, n, &polygon->line_color);
+  renderer_ops->draw_polygon(renderer, points, n, &polygon->line_color);
 }
 
-static Object *
+static DiaObject *
 polygon_create(Point *startpoint,
 	       void *user_data,
 	       Handle **handle1,
@@ -312,13 +249,10 @@ polygon_create(Point *startpoint,
 {
   Polygon *polygon;
   PolyShape *poly;
-  Object *obj;
+  DiaObject *obj;
   Point defaultx = { 1.0, 0.0 };
   Point defaulty = { 0.0, 1.0 };
 
-  init_default_values();
-
-  /*polygon_init_defaults();*/
   polygon = g_new0(Polygon, 1);
   poly = &polygon->poly;
   obj = &poly->object;
@@ -335,7 +269,7 @@ polygon_create(Point *startpoint,
     poly->points[2] = *startpoint;
     point_add(&poly->points[2], &defaulty);
   } else {
-    PolygonCreateData *pcd = (PolygonCreateData *)user_data;
+    MultipointCreateData *pcd = (MultipointCreateData *)user_data;
     
     polyshape_init(poly, pcd->num_points);
     polyshape_set_points(poly, pcd->num_points, pcd->points);
@@ -361,12 +295,12 @@ polygon_destroy(Polygon *polygon)
   polyshape_destroy(&polygon->poly);
 }
 
-static Object *
+static DiaObject *
 polygon_copy(Polygon *polygon)
 {
   Polygon *newpolygon;
   PolyShape *poly, *newpoly;
-  Object *newobj;
+  DiaObject *newobj;
 
   poly = &polygon->poly;
  
@@ -391,7 +325,7 @@ static void
 polygon_update_data(Polygon *polygon)
 {
   PolyShape *poly = &polygon->poly;
-  Object *obj = &poly->object;
+  DiaObject *obj = &poly->object;
   ElementBBExtras *extra = &poly->extra_spacing;
 
   polyshape_update_data(poly);
@@ -434,12 +368,12 @@ polygon_save(Polygon *polygon, ObjectNode obj_node,
   
 }
 
-static Object *
+static DiaObject *
 polygon_load(ObjectNode obj_node, int version, const char *filename)
 {
   Polygon *polygon;
   PolyShape *poly;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
 
   polygon = g_malloc0(sizeof(Polygon));
@@ -488,7 +422,7 @@ polygon_load(ObjectNode obj_node, int version, const char *filename)
 }
 
 static ObjectChange *
-polygon_add_corner_callback (Object *obj, Point *clicked, gpointer data)
+polygon_add_corner_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Polygon *poly = (Polygon*) obj;
   int segment;
@@ -502,7 +436,7 @@ polygon_add_corner_callback (Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-polygon_delete_corner_callback (Object *obj, Point *clicked, gpointer data)
+polygon_delete_corner_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Handle *handle;
   int handle_nr, i;
